#!/usr/bin/env python3
# Adapted from an idea at https://pymotw.com/2/Queue/index.html#module-Queue
import requests
import json
import threading
import queue  # replaces import Queue in python 2.x

def weather_report(url: str, location: str):
    params = {"q" : location , "units": "imperial", "appid": "f4aa6305ea8940b3f24bb23c6b0fe4c3"}
    rq = requests.get(url, params)
    jd = json.loads(rq.text)
    high_temp = float(jd['main']['temp_max'])
    low_temp = float(jd['main']['temp_min'])
    return location, low_temp, high_temp

def worker(url: str, input_queue: queue.Queue(), output_queue: queue.Queue()):
    while not input_queue.empty():
        try:
            location = input_queue.get(block=True, timeout=1)
        except queue.Empty:
            return
        weather_tuple = weather_report(url, location)
        output_queue.put(weather_tuple)
        input_queue.task_done()

def main():
    threads = []
    num_threads = 3
    url = "http://api.openweathermap.org/data/2.5/weather"
    input_queue = queue.Queue()
    output_queue = queue.Queue()
    for city in ['Detroit', 'London', 'Chicago', 'Miami', 'Prague',
                 'Ann Arbor', 'Mackinac Island', 'Hell,MI',
                 'Paradise,MI', 'Misery Bay,MI','Bad Axe,MI']:
        input_queue.put(city)

    for i in range(num_threads):
        t = threading.Thread(target=worker, args=(url, input_queue, output_queue))
        threads.append(t)
        t.start()

    print('*** Main thread waiting')
    input_queue.join()
    print('*** Done')
    while not output_queue.empty():
        (location, low_temp, high_temp) = output_queue.get()
        print("%-20s: Low %3.1fF, High %3.1fF" % (location, low_temp, high_temp))

if __name__ == "__main__":
    main()
